#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Tech - expressions régulières
# 
# Les [expressions régulières](https://fr.wikipedia.org/wiki/Expression_r%C3%A9guli%C3%A8re) sont utilisées pour rechercher des motifs dans un texte tel que des mots, des dates, des nombres...

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Enoncé
# 
# Le texte suivant est un poème d'Arthur Rimbaud, Les Voyelles. On veut en extraire tous les mots.

# In[2]:


poeme = """
A noir, E blanc, I rouge, U vert, O bleu, voyelles,
Je dirai quelque jour vos naissances latentes.
A, noir corset velu des mouches éclatantes
Qui bombillent autour des puanteurs cruelles,

Golfe d'ombre; E, candeur des vapeurs et des tentes,
Lance des glaciers fiers, rois blancs, frissons d'ombelles;
I, pourpres, sang craché, rire des lèvres belles
Dans la colère ou les ivresses pénitentes;

U, cycles, vibrements divins des mers virides,
Paix des pâtis semés d'animaux, paix des rides
Que l'alchimie imprime aux grands fronts studieux;

O, suprême clairon plein de strideurs étranges,
Silences traversés des Mondes et des Anges:
—O l'Oméga, rayon violet de Ses Yeux!
"""


# ### Exercice 1 : utiliser les expression régulières pour extraire tous les mots
# 
# En python, il faut utiliser le module [re](https://docs.python.org/3/library/re.html). Il faudra lire le paragraphe sur la syntaxe [Regular Expression Syntax](https://docs.python.org/3/library/re.html#regular-expression-syntax). Autres lectures : [Expressions régulières](http://www.xavierdupre.fr/app/teachpyx/helpsphinx/c_regex/regex.html#a-quoi-ca-sert).

# In[3]:


def extract_words(text):
    # utiliser les exrp
    pass
    
extract_words(poeme)


# ### Exercice 2 : utiliser les expression régulières pour extraire tous les mots se terminant par la lettre s

# In[4]:





# ### Exercice 3 : utiliser les expression régulières pour remplacer tous les "de" en 2
# 
# Les fonctions [finditer](https://docs.python.org/3/library/re.html#re.finditer) ou [sub](https://docs.python.org/3/library/re.html#re.sub) pourraient vous être utile.

# In[5]:





# ### Exercice 4 : utiliser les expression régulières pour extraire les lignes des rimes en `elle` ou `elles` ou `aile` ou `ailes`
# 
# La fonction [finditer](https://docs.python.org/3/library/re.html#re.finditer) pourrait vous être utile.

# In[6]:





# ## Réponses

# ### Exercice 1 : utiliser les expression régulières pour extraire tous les mots
# 
# Les accents sont traités comme des lettres différentes par les expressions régulières. On peut soit les garder, soit les remplacer. Pour ce faire, on peut lire [What is the best way to remove accents (normalize) in a Python unicode string?](https://stackoverflow.com/questions/517923/what-is-the-best-way-to-remove-accents-normalize-in-a-python-unicode-string).

# In[7]:


import unicodedata

def strip_accents(s):
   return ''.join(c for c in unicodedata.normalize('NFD', s)
                  if unicodedata.category(c) != 'Mn')

strip_accents('têtu')


# In[8]:


import re

def extract_words(text):
    text_sans_accent = strip_accents(text)
    return re.findall('[A-Za-z]+', text_sans_accent)
    
mots = extract_words(poeme)
mots[:5]


# ### Exercice 2 : utiliser les expression régulières pour extraire tous les mots se terminant par la lettre s
# 
# On modifie le motif pour qu'il se termine par la lettre *s*. Le caractère ``\b`` est utilisé pour signifier que cette lettre ne peut se trouver qu'à la fin d'un mot.

# In[9]:


def extract_words_lettre(text, lettre='s'):
    text_sans_accent = strip_accents(text)
    return re.findall('[A-Za-z]+[' + lettre + ']\\b', 
                      text_sans_accent)
    
mots = extract_words_lettre(poeme, 'se')
mots[:5]


# ### Exercice 3 : utiliser les expression régulières pour remplacer tous les "de" en 2

# In[10]:


re.sub("de\\b", "2", poeme)


# ### Exercice 4 : utiliser les expression régulières pour extraire les lignes des rimes en `elle` ou `elles` ou `aile` ou `ailes`
# 
# Un petit essai avant la solution.

# In[11]:


re.findall("(((aile)|(elle))s?\\b)", poeme)


# Un autre pour se convaincre...

# In[12]:


for m in re.finditer("(((aile)|(elle))s?\\b)", poeme):
    print('%02d-%02d: %s' % (
        m.start(), m.end(), m.group(0)))


# On mélange. On découpe en ligne d'abord, et on applique le même traitement sur chaque ligne.

# In[13]:


for i, ligne in enumerate(poeme.split('\n')):
    for m in re.finditer("(((aile)|(elle))s?\\b)", ligne):
        print('% 2d: %02d-%02d/%02d: %s' % (
            i + 1, m.start(), m.end(), len(ligne), ligne))


# Il ne resterait plus qu'à vérifier que la rime trouvée, le motif, se trouve à la fin de la ligne.

# In[14]: